__author__ = 'dexter'

# -*- coding: utf-8 -*-
"""
Created on Mon Oct  6 22:34:42 2014

@author: dexter pratt
"""
import logging
import json


logger = logging.getLogger(__name__)

# ---------------------
# Misc utility functions
#---------------------


def blankNdexNetwork():
    return {"type": "Network",
            "namespaces": {},
            "nodes": {},
            "edges": {},
            "properties": [],
            "supports": {},
            "citations": {},
            "functionTerms": {},
            "baseTerms": {},
            "reifiedEdgeTerms": {}
    }


def blankNdexNode():
    return {"type": "Node",
            "aliases": [],
            "relatedTerms": [],
            "properties": [],
            "presentationProperties": [],
            "supportIds": []
    }


def blankNdexEdge():
    return {"type": "Edge",
            "citationIds": [],
            "presentationProperties": [],
            "properties": [],
            "supportIds": []
    }


def blankNdexBaseTerm():
    return {"type": "BaseTerm",
            "termType": "BaseTerm",
    }


def blankNdexFunctionTerm():
    return {"type": "FunctionTerm",
            "parameterIds": [],
            "termType": "FunctionTerm",
    }


def blankNdexReifiedEdgeTerm():
    return {"type": "ReifiedEdgeTerm",
            "termType": "ReifiedEdgeTerm",
    }


def blankNdexCitation():
    return {"type": "Citation",
            "contributors": [],
            "edges": [],
            "nodes": [],
            "presentationProperties": [],
            "properties": [],
    }


def blankNdexSupport():
    return {"type": "Support",
            "edges": [],
            "nodes": [],
            "presentationProperties": [],
            "properties": [],
    }


def blankNdexNamespace():
    return {"type": "Namespace",
            "presentationProperties": [],
            "properties": [],
    }


def writeAspectElements(aspectName, elements, out):
    jsonString = json.dumps({aspectName: elements})
    out.write(jsonString + "\n")


#---------------------
# CX Converter Class
#---------------------

class Cx2NdexConverter:
    def __init__(self, cx_aspect_fragment_array):
        self.functionTermBlankNodeIds = {}
        self.cx_aspect_fragments = cx_aspect_fragment_array
        self.ndex_network = blankNdexNetwork()
        self.out = None
        self.nextId = 1000
        self.prefixNamespaceIdMap = {}
        self.cx2NdexIdMap = {}
        self.termStringToIdMap = {}

    def newId(self, cx_id=None):
        id = self.nextId
        self.nextId = self.nextId + 1
        if cx_id:
            self.cx2NdexIdMap[cx_id] = id
        return id

    def getNdexId(self, cx_id, create=False):
        ndex_id = self.cx2NdexIdMap.get(cx_id)
        if not ndex_id:
            if create:
                ndex_id = self.newId(cx_id)
        return ndex_id

    def get_ndex_namespace_id_by_prefix(self, prefix, create=False):
        # do we already have a namespace corresponding to this prefix?
        ndex_namespace_id = self.prefixNamespaceIdMap.get(prefix)

        if create and not ndex_namespace_id:
            # create a namespace corresponding to the prefix
            ndex_namespace = blankNdexNamespace()
            ndex_namespace["prefix"] = prefix
            # get an id for the new namespace,  store in the map of prefix->namespace id
            ndex_namespace_id = self.newId()
            ndex_namespace["id"] = ndex_namespace_id

            self.ndex_network['namespaces'][str(ndex_namespace_id)] = ndex_namespace
            self.prefixNamespaceIdMap[prefix] = ndex_namespace_id

        return ndex_namespace_id

    def get_ndex_node_id_by_cx_id(self, cx_id, create=False):
        ndex_node_id = self.getNdexId(cx_id)
        if create and not ndex_node_id:
            ndex_node_id = self.getNdexId(cx_id, True)
            # create a node entry with no properties
            ndex_node = blankNdexNode()
            ndex_node['id'] = ndex_node_id
            self.ndex_network['nodes'][str(ndex_node_id)] = ndex_node
        return ndex_node_id

    def get_ndex_edge_id_by_cx_id(self, cx_id, create=False):
        ndex_edge_id = self.getNdexId(cx_id)
        if create and not ndex_edge_id:
            ndex_edge_id = self.getNdexId(cx_id, True)
            ndex_edge = blankNdexEdge()
            ndex_edge['id'] = ndex_edge_id
            self.ndex_network['edges'][str(ndex_edge_id)] = ndex_edge
        return ndex_edge_id

    def get_ndex_function_term_id_by_cx_id(self, cx_id, create=False):
        ndex_function_term_id = self.getNdexId(cx_id)
        if create and not ndex_function_term_id:
            ndex_function_term_id = self.getNdexId(cx_id, True)
            ndex_function_term = blankNdexFunctionTerm()
            ndex_function_term['id'] = ndex_function_term_id
            self.ndex_network['functionTerms'][str(ndex_function_term_id)] = ndex_function_term
        return ndex_function_term_id

    def get_ndex_base_term_id_for_string(self, term_string):

        ndex_base_term_id = self.termStringToIdMap.get(term_string)
        if ndex_base_term_id:
            return ndex_base_term_id

        else:
            # have not created this base term yet
            # parse term string to get prefix and identifier, split at first colon
            ndex_base_term = blankNdexBaseTerm()
            ndex_base_term_id = self.newId()
            ndex_base_term['id'] = ndex_base_term_id
            prefix, colon, identifier = term_string.partition(':')
            if colon == "":
                # no colon found, so treat the whole as the name in the base term, no namespace
                ndex_base_term['name'] = term_string

            else:
                ndex_namespace_id = self.get_ndex_namespace_id_by_prefix(prefix, True)
                ndex_base_term['namespaceId'] = ndex_namespace_id
                ndex_base_term['name'] = identifier

            # because we are creating the base term,
            # get an id for the new baseterm,
            # then store in the map where termstring->base term id

            self.termStringToIdMap[term_string] = ndex_base_term_id
            self.ndex_network['baseTerms'][str(ndex_base_term_id)] = ndex_base_term
        return ndex_base_term_id

    def get_ndex_citation_id_by_cx_id(self, cx_id, create=False):
        ndex_citation_id = self.getNdexId(cx_id)
        if create and not ndex_citation_id:
            ndex_citation_id = self.getNdexId(cx_id, True)
            ndex_citation = blankNdexCitation()
            ndex_citation['id'] = ndex_citation_id
            self.ndex_network['citations'][str(ndex_citation_id)] = ndex_citation

        return ndex_citation_id

    def get_ndex_support_id_by_cx_id(self, cx_id, create=False):
        ndex_support_id = self.getNdexId(cx_id)
        if create and not ndex_support_id:
            ndex_support_id = self.getNdexId(cx_id, True)
            ndex_support = blankNdexSupport()
            ndex_support['id'] = ndex_support_id
            self.ndex_network['supports'][str(ndex_support_id)] = ndex_support
        return ndex_support_id

    def get_ndex_reified_edge_term_id_by_ndex_edge_id(self, ndex_edge_id, create=False):
        # search the reified edge terms
        for reified_edge_term_id, term in self.ndex_network.get('reifiedEdgeTerms'):
            if term.get("edgeId") == ndex_edge_id:
                return reified_edge_term_id
        # didn't find it, so create
        id = self.newId()
        reified_edge_term = blankNdexReifiedEdgeTerm()
        reified_edge_term['id'] = id
        reified_edge_term['edgeId'] = ndex_edge_id
        self.ndex_network['reifiedEdgeTerms'][str(id)] = reified_edge_term
        return id

    def convertToNdex(self):
        for aspect_fragment in self.cx_aspect_fragments:
            self.handle_aspect_fragment(aspect_fragment)

        return self.ndex_network

    def handle_aspect_fragment(self, aspect_fragment):
        # dispatch on type of aspect_fragment
        # ignore any that are unknown
        keys = aspect_fragment.keys()
        type = keys[0]
        elements = aspect_fragment.get(type)
        for element in elements:
            if type == "@context":
                self.handle_context_aspect_fragment(element)
            elif type == "nodes":
                self.handle_nodes_aspect_fragment(element)
            elif type == "edges":
                self.handle_edges_aspect_fragment(element)
            elif type == "edgeIdentities":
                self.handle_edge_identities_aspect_fragment(element)
            elif type == "functionTerms":
                self.handle_function_terms_aspect_fragment(element)
            elif type == "nodeIdentities":
                self.handle_node_identities_aspect_fragment(element)
            elif type == "citations":
                self.handle_citations_aspect_fragment(element)
            elif type == "supports":
                self.handle_supports_aspect_fragment(element)
            elif type == "profile":
                self.handle_profile_aspect_fragment(element)
            elif type == "networkProperties":
                self.handle_network_properties_aspect_fragment(element)
            elif type == "elementProperties":
                self.handle_element_properties_aspect_fragment(element)
            elif type == "reifiedEdges":
                self.handle_reified_edges_aspect_fragment(element)
            else:
                logger.error("Unknown aspect_fragment type: " + type)

    def handle_context_aspect_fragment(self, element):
        # expecting {prefix : uri}
        # references to the namespace by prefix in previously processed aspect_fragments may already have created it.
        # but we use a method that will create it if it has not been created yet.
        prefix = element.keys()[0]
        ndex_namespace_id = self.get_ndex_namespace_id_by_prefix(prefix, True)
        ndex_namespace = self.ndex_network['namespaces'].get(str(ndex_namespace_id))

        uri = element.values()[0]
        if uri:
            ndex_namespace["uri"] = uri

    def handle_nodes_aspect_fragment(self, element):
        # expecting {"@id" : blankNodeId}
        self.get_ndex_node_id_by_cx_id(element.get('@id'), True)

    def handle_edges_aspect_fragment(self, element):
        # {"@id" : blankNodeId, "source" : blankNodeId, "target" : blankNodeId)}
        ndex_edge_id = self.get_ndex_edge_id_by_cx_id(element.get('@id'), True)
        ndex_edge = self.ndex_network['edges'].get(str(ndex_edge_id))
        cx_source_node_id = element.get('source')
        if cx_source_node_id:
            ndex_source_node_id = self.get_ndex_node_id_by_cx_id(cx_source_node_id)
            ndex_edge['subjectId'] = ndex_source_node_id

        cx_target_node_id = element.get('target')
        if cx_target_node_id:
            ndex_target_node_id = self.get_ndex_node_id_by_cx_id(cx_target_node_id)
            ndex_edge['objectId'] = ndex_target_node_id

    def handle_edge_identities_aspect_fragment(self, element):
        # {"edge" : blankNodeId, "relationship" : baseTermString}
        cx_edge_id = element.get('edge')
        predicate_string = element.get('relationship')
        if cx_edge_id and predicate_string:
            ndex_edge_id = self.get_ndex_edge_id_by_cx_id(cx_edge_id, True)
            ndex_edge = self.ndex_network['edges'].get(str(ndex_edge_id))
            ndex_base_term_id = self.get_ndex_base_term_id_for_string(predicate_string)
            ndex_edge['predicateId'] = ndex_base_term_id

    def handle_function_terms_aspect_fragment(self, element):
        # expecting {"@id" : function_term_id, "function" : cx_term_string, "parameters" : [parameters]}
        cx_function_term_id = element.get("@id")
        ndex_function_term_id = self.get_ndex_function_term_id_by_cx_id(element.get('@id'), True)
        ndex_function_term = self.ndex_network['functionTerms'].get(str(ndex_function_term_id))

        cx_function_term_string = element.get("function")
        ndex_function_id = self.get_ndex_base_term_id_for_string(cx_function_term_string)
        ndex_function_term['functionTermId'] = ndex_function_id

        cx_parameters = element.get("parameters")

        ndex_parameter_ids = []
        for cx_parameter in cx_parameters:
            # each parameter is either a cx_term_string OR a blank node id for a function term
            ndex_parameter_id, term_type = self.get_ndex_id_by_cx_id_or_term_string(cx_parameter)
            ndex_parameter_ids.append(ndex_parameter_id)

        ndex_function_term['parameterIds'] = ndex_parameter_ids


    # BUG: needs to distinguish CX blank nodes that are reified edges.
    # For those, we need to create the reified edge term and return that ndex id
    def get_ndex_id_by_cx_id_or_term_string(self, cx_parameter):
        if cx_parameter.startswith("_"):
            # its a blank node id
            return self.get_ndex_function_term_id_by_cx_id(cx_parameter, True), 'functionTerm'
        else:
            return self.get_ndex_base_term_id_for_string(cx_parameter), 'baseTerm'

    def handle_node_identities_aspect_fragment(self, element):
        # expecting {node: cx_node_id, represents: item, aliases: [items...]}
        ndex_node_id = self.get_ndex_node_id_by_cx_id(element.get("node"), True)
        ndex_node = self.ndex_network['nodes'].get(str(ndex_node_id))
        if "represents" in element:
            ndex_represents_id, term_type = self.get_ndex_id_by_cx_id_or_term_string(element['represents'])
            ndex_node["represents"] = ndex_represents_id
            ndex_node["representsTermType"] = term_type

        cx_aliases = element.get('alias')
        if cx_aliases:
            ndex_aliases = []
            for cx_alias in element.get('alias'):
                alias, term_type = self.get_ndex_id_by_cx_id_or_term_string(cx_alias)
                ndex_aliases.append(alias)
            if len(ndex_aliases) < 0:
                ndex_node['alias'] = ndex_aliases

        cx_related_terms = element.get('relatedTerms')
        if cx_related_terms:
            ndex_related_terms = []
            for cx_term in element.get('relatedTerms'):
                related_term, term_type = self.get_ndex_id_by_cx_id_or_term_string(cx_term)
                ndex_related_terms.append(related_term)
            if len(ndex_related_terms) < 0:
                ndex_node['relatedTerms'] = ndex_related_terms

        if "name" in element:
            ndex_node['name'] = element.get('name')

    def handle_reified_edges_aspect_fragment(self, element):
        ndex_node_id = self.get_ndex_node_id_by_cx_id(element.get("node"), True)
        ndex_edge_id = self.get_ndex_edge_id_by_cx_id(element.get("edge"), True)
        reified_edge_term_id = self.get_ndex_reified_edge_term_id_by_ndex_edge_id(ndex_edge_id, True)
        ndex_node = self.ndex_network['nodes'].get(str(ndex_node_id))
        ndex_node['represents'] = reified_edge_term_id
        ndex_node['representsTermType'] = 'reifiedEdgeTerm'

    def handle_citations_aspect_fragment(self, element):
        ndex_citation_id = self.get_ndex_citation_id_by_cx_id(element.get('@id'), True)
        ndex_citation = self.ndex_network['citations'].get(str(ndex_citation_id))
        for key, value in element.iteritems():
            if key == '@id':
                continue
            elif key == 'nodes':
                for cx_id in value:
                    ndex_node_id = self.get_ndex_node_id_by_cx_id(cx_id, True)
                    ndex_node = self.ndex_network['nodes'].get(str(ndex_node_id))
                    if not 'citationIds' in ndex_node:
                        ndex_node['citationIds'] = []
                    ndex_node['citationIds'].append(ndex_citation_id)

            elif key == 'edges':
                for cx_id in value:
                    ndex_edge_id = self.get_ndex_edge_id_by_cx_id(cx_id, True)
                    ndex_edge = self.ndex_network['edges'].get(str(ndex_edge_id))
                    if not 'citationIds' in ndex_edge:
                        ndex_edge['citationIds'] = []
                    ndex_edge['citationIds'].append(ndex_citation_id)

            else:
                ndex_citation[key] = value

    def handle_supports_aspect_fragment(self, element):
        ndex_support_id = self.get_ndex_support_id_by_cx_id(element.get('@id'), True)
        ndex_support = self.ndex_network['supports'].get(str(ndex_support_id))
        for key, value in element.iteritems():
            if key == '@id':
                continue
            elif key == 'nodes':
                for cx_id in value:
                    ndex_node_id = self.get_ndex_node_id_by_cx_id(cx_id, True)
                    ndex_node = self.ndex_network['nodes'].get(str(ndex_node_id))
                    if not 'supportIds' in ndex_node:
                        ndex_node['supportIds'] = []
                    ndex_node['supportIds'].append(ndex_support_id)

            elif key == 'edges':
                for cx_id in value:
                    ndex_edge_id = self.get_ndex_edge_id_by_cx_id(cx_id, True)
                    ndex_edge = self.ndex_network['edges'].get(str(ndex_edge_id))
                    ndex_edge_supports = ndex_edge.get('supports')
                    if not 'supportIds' in ndex_edge:
                        ndex_edge['supportIds'] = []
                    ndex_edge['supportIds'].append(ndex_support_id)
            elif key == 'citation':
                ndex_support[key] = self.get_ndex_citation_id_by_cx_id(value, True)
            else:
                ndex_support[key] = value

    def handle_profile_aspect_fragment(self, element):
        for key, value in element.iteritems():
            if key == "dc:title":
                self.ndex_network['name'] = value
            if key == "dc:description":
                self.ndex_network['description'] = value

    def handle_network_properties_aspect_fragment(self, element):
        self.ndex_network['properties'].append(element)

    def handle_element_properties_aspect_fragment(self, element):
        predicateString = element.get('property')
        value = element.get('value')
        predicateId = self.get_ndex_base_term_id_for_string(predicateString)
        ndex_property = {"predicateString": predicateString,
                         "predicateId" : predicateId,
                         "value": value,
                         "type" : "NdexPropertyValuePair"}
        if "edge" in element:
            ndex_edge_id = self.get_ndex_edge_id_by_cx_id(element.get('edge'), True)
            ndex_edge = self.ndex_network['edges'].get(str(ndex_edge_id))
            if "properties" in ndex_edge:
                ndex_edge['properties'].append(ndex_property)
            else:
                ndex_edge['properties'] = [ndex_property]
        elif "node" in element:
            ndex_node_id = self.get_ndex_node_id_by_cx_id(element.get('node'), True)
            ndex_node = self.ndex_network['nodes'].get(str(ndex_node_id))
            if "properties" in ndex_node:
                ndex_node['properties'].append(ndex_property)
            else:
                ndex_node['properties'] = [ndex_property]    







import json
import csv
from os import path
import jsonschema
import logging
from ndexutil.exceptions import NDExUtilError

version = "0.1"

logger = logging.getLogger(__name__)


class CXStreamWriter:
    """Writes CX data to stream
    """

    def __init__(self, f):
        """
        Constructor
        :param f: Output stream
        """
        self._outputstream = f
        #  0 -- begining of a stream
        #  1 -- after premetadata, outside a fragment
        #  2 -- after postmetadata
        self._state = 0  # beginning of a stream

    def write_pre_metadata(self, metadata):
        """
        Writes the pre meta data aspect
        :param metadata:
        :raises NdexUtilError: if write_aspect_fragment or
                               write_post_metadata has already been called or
                               if output stream set in constructor is None
        :return: None
        """
        if self._outputstream is None:
            raise NDExUtilError("Output stream is None")
        if self._state != 0:
            raise NDExUtilError("PreMetadata has already been written, you can only write it once.")
        self._outputstream.write('[')
        json.dump({"numberVerification": [{"longNumber": 281474976710655}]}, self._outputstream)
        self._outputstream.write(',')
        json.dump({"metaData": metadata}, self._outputstream)
        self._outputstream.write(',\n')
        self._outputstream.flush()
        self._state = 1  # premetadata has been written.

    def write_aspect_fragment(self, fragment):
        """
        Writes aspect fragment.
        If the fragment have values of type list_of_double,
        list_of_boolean, list_of_long or list_of_integer
        the values in the list need to be quoted
        :param fragment: Fragment as list or dict to convert to JSON via json.dump
        :raises NdexUtilError: if write_pre_metadata has not been called first
        :return: None
        """
        if self._state != 1:
            raise NDExUtilError("Data aspects can only be written between PreMetadata and PostMetadata.")

        json.dump(fragment, self._outputstream)
        self._outputstream.write(',')

    def write_post_metadata(self, metadata):
        """
        Writes the post meta data aspect. Once this is called this object
        can no longer be used cause all methods will raise an error.
        :param metadata:
        :return:
        """
        if self._state != 1:
            raise NDExUtilError("Post metadata aspect can only be written after PreMetadata and data aspects.")
        json.dump({"metaData": metadata}, self._outputstream)
        self._outputstream.write(',')
        json.dump({"status": [{"error": "", "success": True}]}, self._outputstream)
        self._outputstream.write(']')
        self._outputstream.flush()
        self._state = 2


class StreamTSVLoader (object):
    """
    Stream based TSV Loader
    """

    def __init__(self, loading_plan_file, style_cx):
        """
        Constructor that loads and validates the loading_plan_file as well as extracts
        the style from the style_cx object
        :param loading_plan_file: Path to loading plan file
        :param style_cx: NiceCXNetwork object containing a style 'cyVisualProperties' as
                         an opaque aspect
        """

        # fullpath of the loading plan json file
        # style CX object (niceCX object)
        # _plan is the full loading plan
        # _sytle_cx is a niceCx object which has the style we are going to use in this loader.
        # read and validate the plan
        # open the schema first
        if style_cx:
            cy_visual_properties = style_cx.get_opaque_aspect("cyVisualProperties")
            if not cy_visual_properties:
                raise NDExUtilError("cyVisualProperties aspect is missing in style template CX.")
            if len(cy_visual_properties) != 3:
                raise NDExUtilError("cyVisualProperties in style template has " +
                                    str(len(cy_visual_properties)) + " elements. It should be 3.")
            self._visual_properties_aspect = cy_visual_properties
        else:
            self._visual_properties_aspect = None

        here = path.abspath(path.dirname(__file__))
        with open(path.join(here, 'loading_plan_schema.json')) as json_file:
            self._plan_schema = json.load(json_file)

        with open(loading_plan_file, 'r') as lp:
            self._plan = json.load(lp)

        try:
            jsonschema.validate(self._plan, self._plan_schema)

        except jsonschema.ValidationError as e1:
            print("Failed to parse the loading plan: " + e1.message)
            print('at path: ' + str(e1.absolute_path))
            print("in block: ")
            print(e1.instance)
            logger.exception(e1)
            raise NDExUtilError("Malformed TSV loading plan: " + str(e1.absolute_path) + ' : ' + str(e1))

    def write_cx_network(self, tsv_file_discriptor, output_file_descriptor,
                         network_attributes = None, batchsize=20000):
        """
        Both input and output descriptor as objects NOT file names.
        this function is not thread safe
        caller need to close the input and output stream after this function is executed.
        the attributes in the networkAttributes parameter need to be in the format
        of cx. eg: quoted values in list.

        :param tsv_file_discriptor:
        :param output_file_descriptor:
        :param network_attributes:
        :param batchsize:
        :return:
        """
        # initialize the environment
        self.batchsize = batchsize

        # table to track the node constructed in this network
        # key: the external id of the node. Can come from represent or node name depend on the loading plan
        # value: node and its attributes.
        self.nodeTable = {}
        self.nodeCounter = 0
        self.edgeCounter = 0
        self.nodeAttrCounter = 0
        self.edgeAttrCounter = 0
        self.newNodes = []  # new nodes in the batch, each element has nodes and nodesAttribute info
        self.newEdges = []  # new edges in the batch, each element has edges and edgesAttribute info

        # start the process
        header = [h.strip() for h in tsv_file_discriptor.readline().split('\t')]
        self._check_header_vs_plan(header)

        # initialize the writer
        self.cxWriter = CXStreamWriter(output_file_descriptor)

        # write the conext as network attribute
        net_attrs = []
        context = self._plan.get("context")
        if context:
            net_attrs.append({"n": "@context", "v": json.dumps(context)})
        if network_attributes:
            if type(network_attributes) is list:
                net_attrs.extend(network_attributes)
            else:
                net_attrs.append(network_attributes)

        # prepare metadata
        premetadata = [{
                "name": "nodes",
                "version": "1.0",
                "consistencyGroup": 1
            }, {
                "name": "edges",
                "version": "1.0",
                "consistencyGroup": 1
            }
            ]

        if self._plan.get("source_plan").get("property_columns") or self._plan.get("target_plan").get("property_columns"):
            premetadata.append({"name": "nodeAttributes",
                                "version": "1.0",
                                "consistencyGroup": 1})

        if self._plan.get("edge_plan").get("property_columns"):
            premetadata.append({
                                "name": "edgeAttributes",
                                "version": "1.0",
                                "consistencyGroup": 1
            })

        if net_attrs:
            premetadata.append({"name": "networkAttributes", "version": "1.0", "consistencyGroup": 1,
                                "elementCount": len(net_attrs)})

        if self._visual_properties_aspect:
            premetadata.append({"consistencyGroup": 1, "elementCount": 3, "name": "cyVisualProperties", "version": "1.0"})

        self.cxWriter.write_pre_metadata(premetadata)

        # write the network attr and styles
        if net_attrs:
            self.cxWriter.write_aspect_fragment({"networkAttributes": net_attrs})

        if self._visual_properties_aspect:
            self.cxWriter.write_aspect_fragment({"cyVisualProperties": self._visual_properties_aspect})

        # start processing the file
        reader = csv.DictReader(tsv_file_discriptor, dialect='excel-tab', fieldnames=header)
        row_count = 2
        # for debugging purposes, max_rows can be set so that only a subset of a large file is processed
        for row in reader:
            try:
                self._process_row(row)
                row_count = row_count + 1
            except RuntimeError as err1:
                print("Error occurred in line " + str(row_count) + ". Message: " + str(err1))
                raise err1
            except Exception as err2:
                print("Error occurred in line " + str(row_count) + ". Message: " + str(err2))
                raise err2

        # flush out whats left in the buffer
        self._print_batch()

        # write the post metadata and finish the writing

        postmetadata = [{"name": "nodes", "idCounter": self.nodeCounter, "elementCount": self.nodeCounter},
                        {"name": "edges", "idCounter": self.edgeCounter, "elementCount": self.edgeCounter},
                        {"name": "edgeAttributes", "elementCount": self.edgeAttrCounter},
                        {"name": "nodeAttributes", "elementCount": self.nodeAttrCounter}]

        self.cxWriter.write_post_metadata(postmetadata)

    def _check_header_vs_plan(self, header):
        # each column name referenced in the plan must be in the header, otherwise raise an exception
        StreamTSVLoader._check_column(self._plan.get('source_plan').get('rep_column'), header)
        StreamTSVLoader._check_column(self._plan.get('source_plan').get('node_name_column'), header)
        StreamTSVLoader._check_plan_property_columns(self._plan.get('source_plan'), header)

        StreamTSVLoader._check_column(self._plan.get('target_plan').get('rep_column'), header)
        StreamTSVLoader._check_column(self._plan.get('target_plan').get('node_name_column'), header)
        StreamTSVLoader._check_plan_property_columns(self._plan.get('target_plan'), header)

        StreamTSVLoader._check_column(self._plan.get('edge_plan').get('predicate_id_column'), header)
        StreamTSVLoader._check_column(self._plan.get('edge_plan').get('citation_id_column'), header)
        StreamTSVLoader._check_plan_property_columns(self._plan.get('edge_plan'), header)

    @staticmethod
    def _check_column(column_name, header):
        if column_name:
            if column_name not in header:
                raise Exception("Error in import plan: column name " + column_name +
                                " in import plan is not in header " + str(header))

    @staticmethod
    def _check_plan_property_columns(column_name_raw, header):
        if column_name_raw:
            if type(column_name_raw) is str:
                col_list = column_name_raw.split("::")
                if len(col_list) > 2:
                    raise Exception("Column name '" + column_name_raw + "' has too many :: in it")
                else:
                    column_name = col_list[0]
                    if column_name not in header:
                        raise Exception(
                            "Error in import plan: column name " + column_name + " in import plan is not in header " +
                            str(header))
            else:
                column_name = column_name_raw.get("column_name")
                if column_name and (column_name not in header):
                    raise Exception(
                        "Error in import plan: column name " + column_name + " in import plan is not in header " +
                        str(header))

    def _process_row(self, row):
        """
        For each row, we create an edge + edge properties
        For that edge, we may create elements if they are new
        - source node + properties
        - target node + properties
        - predicate term
        :param row:
        :return:
        """

        source_node_id = self._create_node(row, self._plan.get('source_plan'))
        target_node_id = self._create_node(row, self._plan.get('target_plan'))

        self._create_edge(source_node_id, target_node_id, row)

    def _create_node(self, row, node_plan):

        nodename = None

        use_name_as_id = False

        if not node_plan.get('rep_column'):
            use_name_as_id = True

        if use_name_as_id and node_plan.get('rep_prefix'):
            raise RuntimeError("rep_column needs to be defined if re_prefix is defined in your loading plan.")

        if 'node_name_column' in node_plan:
            nodename = row.get(node_plan['node_name_column'])

        if use_name_as_id:
            ext_id = nodename
        else:
            ext_id = row.get(node_plan['rep_column'])

        if not ext_id:
            raise RuntimeError("Id value is missing.")

        node_attr = self._create_attr_obj(node_plan, row)

        if use_name_as_id:
            represent = None
        else:
            represent = (node_plan['rep_prefix'] + ":" + ext_id) if node_plan.get('rep_prefix') else ext_id

        return self._add_node(ext_id, nodename, represent, node_attr)

    def _add_node(self, external_id, node_name, represent, attributes):
        existing_node = self.nodeTable.get(external_id)
        if existing_node:
            # check if everything matches
            if node_name != existing_node.get('n'):
                raise RuntimeError("Node name mismatch on node id " + external_id + ": " +
                                   (node_name if node_name else "''") + " vs " +
                                   existing_node.n if existing_node else "''")
            if existing_node.get('r') != represent:
                raise RuntimeError("Node represent mismatch on node id " + external_id + ": " + (represent if represent else "''")
                                   + " vs " + (existing_node.r if existing_node.r else "''"))

            # check attributes consistency
            tmp_node = {"n": node_name,
                        "r": represent,
                        "attr": attributes}
            tmp_node2 = {"n": existing_node.get("n"), "r": existing_node.get("r"), "attr": existing_node.get("attr")}

            if tmp_node != tmp_node2:
                raise RuntimeError("Node value mismatch between " + json.dumps(tmp_node) + " and " + json.dumps(tmp_node2))
            return existing_node.get('id')
        else:
            new_node = {"id": self.nodeCounter,
                        "n": node_name,
                        "r": represent,
                        "attr": attributes}

            self.nodeCounter += 1
            self.nodeAttrCounter += len(attributes.keys())
            self.nodeTable[external_id] = new_node
            self.newNodes.append(new_node)
            return new_node["id"]

    def _create_attr_obj(self, node_or_edge_plan, row):
        """
        Create attribute object
        :param node_or_edge_plan: node or edge plan in the loading plan
        :param row: current row to be parsed
        :return:
        """
        attr = {}
        if node_or_edge_plan.get('property_columns'):
            for column_raw_temp in node_or_edge_plan['property_columns']:
                if isinstance(column_raw_temp, dict):
                    column_raw = column_raw_temp
                else:
                    if '::' in column_raw_temp:
                        column_split = column_raw_temp.split('::')
                        if len(column_split) > 1:
                            column_raw = {
                                'column_name': column_split[0],
                                'attribute_name': column_split[0],
                                'data_type': column_split[1]
                            }
                        else:
                            column_raw = {
                                'column_name': column_raw_temp,
                                'attribute_name': column_raw_temp
                            }
                    else:
                        column_raw = {
                            'column_name': column_raw_temp,
                            'attribute_name': column_raw_temp
                        }

                if not column_raw.get('data_type'):   # set the default datatype to string
                    column_raw['data_type'] = 'string'

                if not column_raw.get('attribute_name'):  # set the attribute name if it is not defined.
                    column_raw['attribute_name'] = column_raw.get('column_name')

                # type_temp = column_raw.get('data_type')
                value = None

                # this allows us to add arbitary attributes to all source or target nodes
                if column_raw.get('column_name'):
                    value = row.get(column_raw['column_name'])

                if (value is None) and column_raw.get('default_value'):
                    value = column_raw['default_value']

                if value:
                    if column_raw.get('delimiter'):
                        value = value.split(column_raw.get('delimiter'))
                        value = [entry.strip() for entry in value]
                        value = self._data_to_type(value, column_raw['data_type'])
                        # if not type_temp.startswith('list'):
                        #    type_temp = 'list_of_' + type_temp

                        if column_raw.get('value_prefix'):
                            value_list_temp = []
                            for value_item in value:
                                value_temp = column_raw.get('value_prefix') + ":" + str(value_item)
                                value_list_temp.append(value_temp)
                            value = value_list_temp
                    else:
                        value = self._data_to_type(value, column_raw['data_type'])
                        if value is None:
                            return ''

                        if column_raw.get('value_prefix'):
                            value = column_raw.get('value_prefix') + ":" + str(value)

                    attrname = column_raw['attribute_name'] if column_raw.get('attribute_name') \
                        else column_raw['column_name']
                    tmp_attr = {"n": attrname, "v": value}
                    if column_raw['data_type'] != 'string':
                        tmp_attr['d'] = column_raw['data_type']
                    attr[attrname] = tmp_attr

        return attr

    def _data_to_type(self, data, data_type):
        return_data = None

        if type(data) is str:
            data = data.replace('[', '').replace(']', '')
            if 'list_of' in data_type:
                data = data.split(',')

        if data_type == "boolean":
            if type(data) is str:
                return_data = data.lower() == 'true'
            else:
                return_data = bool(data)
        elif data_type == "double" or data_type == "float":
            return_data = float(data)
        elif data_type == "long" or data_type == "integer":
            return_data = int(data)
        elif data_type == "string":
            return_data = str(data)
        elif data_type == "list_of_boolean":
            # Assumption: if the first element is a string then so are the rest...
            if type(data[0]) is str:
                return_data = [s.lower() == 'true' for s in data]
            else:
                return_data = [bool(s) for s in data]
        elif data_type == "list_of_double" or data_type == "list_of_float":
            return_data = [float(s) for s in data]
        elif data_type == "list_of_long" or data_type == "list_of_integer":
            return_data = [int(s) for s in data]
        elif data_type == "list_of_string":
            return_data = [str(s) for s in data]
        else:
            return None

        return return_data

    def _create_edge(self, src_node_id, tgt_node_id, row):
        predicate_str = None
        edge_plan = self._plan.get('edge_plan')
        if edge_plan.get('predicate_id_column'):
            predicate_str = row[edge_plan['predicate_id_column']]

        if not predicate_str and edge_plan.get('default_predicate'):
            predicate_str = edge_plan['default_predicate']

        if not predicate_str:
            raise RuntimeError("Value for predicate string is not found in this row.")
        if edge_plan.get("predicate_prefix"):
            predicate_str = edge_plan['predicate_prefix'] + ":" + predicate_str

        attr = self._create_attr_obj(edge_plan, row)
        new_edge = {"id": self.edgeCounter, "s": src_node_id, "t": tgt_node_id, "i": predicate_str, "attr": attr}
        self.edgeCounter += 1

        self.newEdges.append(new_edge)
        self.edgeAttrCounter += len(attr.keys())

        if len(self.newEdges) >= self.batchsize:
            self._print_batch()

    def _print_batch(self):

        # print new nodes:
        new_nodes = []
        newnode_attrs = []
        for n in self.newNodes:
            new_n = {"@id": n["id"]}
            if n.get("n"):
                new_n['n'] = n.get("n")
            if n.get("r"):
                new_n["r"] = n.get("r")
            new_nodes.append(new_n)
            if n.get("attr"):
                for key, value in n.get("attr").items():
                    tmp_value = value.copy()
                    tmp_value['po'] = n["id"]
                    newnode_attrs.append(tmp_value)

        if new_nodes:
            self.cxWriter.write_aspect_fragment({"nodes": new_nodes})
        if newnode_attrs:
            self.cxWriter.write_aspect_fragment({"nodeAttributes": newnode_attrs})

        # clear the buffer
        self.newNodes.clear()

        # print edges and their attributes
        new_edges = []
        new_edge_attrs = []
        for e in self.newEdges:
            new_e = {'@id': e.get('id'), "s": e.get('s'), "t": e.get('t')}
            if e.get("i"):
                new_e['i'] = e.get("i")
                new_edges.append(new_e)
            if e.get("attr"):
                for key, value in e.get("attr").items():
                    value["po"] = e.get("id")
                    new_edge_attrs.append(value)

        if new_edges:
            self.cxWriter.write_aspect_fragment({"edges": new_edges})

        if new_edge_attrs:
            self.cxWriter.write_aspect_fragment({"edgeAttributes": new_edge_attrs})

        self.newEdges.clear()





import logging
import csv
from os import listdir
from os.path import isfile, join, abspath, dirname, basename, splitext
import ndex.beta.layouts as layouts
import ndex.beta.toolbox as toolbox
from ndex.networkn import NdexGraph
import networkx as nx
import mygene
import json

logger = logging.getLogger(__name__)


NDEX_SIF_INTERACTIONS = ["controls-state-change-of",
                         # First protein controls a reaction that changes the state of the second protein.

                         "controls-transport-of",
                         # First protein controls a reaction that changes the cellular location of the second protein.

                         "controls-phosphorylation-of"  # First protein controls a reaction that changes the phosphorylation status of the second protein.

                         "controls-expression-of",
                         # First protein controls a conversion or a template reaction that changes expression of the second protein.

                         "catalysis-precedes",
                         # First protein controls a reaction whose output molecule is input to another reaction controled by the second protein.

                         "in-complex-with",  # Proteins are members of the same complex.

                         "interacts-with",  # Proteins are participants of the same MolecularInteraction.

                         "neighbor-of",  # Proteins are participants or controlers of the same interaction.

                         "consumption-controled-by",
                         # The small molecule is consumed by a reaction that is controled by a protein

                         "controls-production-of",
                         # The protein controls a reaction of which the small molecule is an output.

                         "controls-transport-of-chemical"  # The protein controls a reaction that changes cellular location of the small molecule.

                         "chemical-affects",  # A small molecule has an effect on the protein state.

                         "reacts-with",  # Small molecules are input to a biochemical reaction.

                         "used-to-produce"  # A reaction consumes a small molecule to produce another small molecule.
                         ]

DIRECTED_INTERACTIONS = ["controls-state-change-of",
                         "controls-transport-of",
                         "controls-phosphorylation-of",
                         "controls-expression-of",
                         "catalysis-precedes",
                         "controls-production-of",
                         "controls-transport-of-chemical",
                         "chemical-affects",
                         "used-to-produce"
                         ]

CONTROL_INTERACTIONS = ["controls-state-change-of",
                        "controls-transport-of",
                        "controls-phosphorylation-of",
                        "controls-expression-of"
                        ]

current_directory = dirname(abspath(__file__))
def get_json_from_file(file_path):
    if(isfile(file_path)):
        c_file = open(file_path, "r")
        c_data = json.load(c_file)
        c_file.close()
        return c_data
    else:
        return None

gene_symbol_mapping = get_json_from_file(join(current_directory, 'gene_symbol_mapping.json'))


def upload_ebs_files(dirpath, ndex, group_id=None, template_network=None, layout=None,
                     update=False, filter=None, max=None, nci_table=None):
    my_layout = _check_layout_(layout)
    my_filter = _check_filter_(filter)
    my_template_network = _check_template_network_(template_network)
    network_id_map = {}
    network_count = 0
    if max is not None:
        logger.info("max files: " + str(max))

    in_dir = []
    in_nci_table = []
    not_in_nci_table = []
    if nci_table:
        for row in nci_table:
            if "Pathway Name" in row:
                in_nci_table.append(row["Pathway Name"])
                #
                # if "Corrected Pathway Name" in row:
                #     in_nci_table.append(row["Corrected Pathway Name"])

    skipped = []
    account_network_map = search_for_non_biopax_networks(ndex)
    files = []
    file_network_names = []
    for file in listdir(dirpath):
        if file.endswith(".sif"):
            files.append(file)
            network_name = network_name_from_path(file)
            file_network_names.append(network_name)

    logger.info(str(len(files)) + 'SIF files to load')
    logger.info(str(len(account_network_map)) + 'Non-Biopax Networks in the account')

    account_networks = account_network_map.keys()
    account_not_file = list(set(account_networks).difference(set(file_network_names)))

    logger.info("%s Networks in the account not in upload files" % (len(account_not_file)))
    for network_name in account_not_file:
        logger.info(" - %s" % (network_name))

    for filename in files:
        network_count = network_count + 1
        if max is not None and network_count > max:
            break

        logger.info("loading ndexebs file #" + str(network_count) + ": " + filename)
        path = join(dirpath, filename)
        network_name = network_name_from_path(path)
        in_dir.append(network_name)

        matching_networks = account_network_map.get(network_name)
        matching_network_count = 0
        if matching_networks and update:
            matching_network_count = len(matching_networks)
            if matching_network_count > 1:
                logger.info("skipping this file because %s existing networks match '%s'" % (len(matching_networks), network_name))
                skipped.append(network_name  + " :duplicate names")
                continue

        ebs = load_ebs_file_to_dict(path)

        if len(ebs) == 0:
            logger.info("skipping this file because no rows were found when processing it as EBS")
            skipped.append(network_name + " :no rows in file")
            continue

        ebs_network = ebs_to_network(ebs, name=network_name)

        if len(ebs_network.nodes()) == 0:
            logger.info("skipping this network because no nodes were found when processing it as EBS")
            skipped.append(network_name + " :no nodes in file")
            continue

        # Do this one first to establish subnetwork and view ids from template
        # this is not ideal, but ok for special case of this loader
        if my_template_network:
            toolbox.apply_network_as_template(ebs_network, template_network)
            logger.info("applied graphic style from " + str(template_network.get_name()))

        if my_filter:
            if filter == "cravat_1":
                cravat_edge_filter(ebs_network)
            if filter == "ndex_1":
                ndex_edge_filter(ebs_network)

        if my_layout:
            if layout == "directed_flow":
                layouts.apply_directed_flow_layout(ebs_network,
                                                   node_width=25,
                                                   use_degree_edge_weights=True,
                                                   iterations=200)
                logger.info("applied directed_flow layout")

        provenance_props = [{"name": "dc:title", "value": network_name}]

        if nci_table:
            add_nci_table_properties(ebs_network, network_name, nci_table, not_in_nci_table)

        ebs_network.set_network_attribute("description", NCI_DESCRIPTION_TEMPLATE % network_name)

        ebs_network.set_network_attribute("organism", "human")

        if nci_table:
            ebs_network.set_network_attribute("version", "27-Jul-2015")

        ebs_network.update_provenance("Created by NDEx EBS network converter", entity_props=provenance_props)

        if update:
            if matching_network_count == 0:
                logger.info("saving new network " + network_name)
                network_url = ndex.save_cx_stream_as_new_network(ebs_network.to_cx_stream())
                network_id = network_url.split("/")[-1]

            elif matching_network_count == 1:
                network_to_update = matching_networks[0]
                logger.info("updating network " + network_to_update.get("name") + " with " + network_name)
                network_id = network_to_update.get("externalId")
                ndex.update_cx_network(ebs_network.to_cx_stream(), network_id)

            else:
                raise ValueError("unexpected case: should not try to update when more than one matching network")
        else:
            logger.info("saving new network " + network_name)
            network_url = ndex.save_cx_stream_as_new_network(ebs_network.to_cx_stream())
            network_id = network_url.split("/")[-1]


        network_id_map[network_name] = network_id

    if group_id:
        logger.info("granting networks to group id " + group_id)
        ndex.grant_networks_to_group(group_id, network_id_map.values())

    for network_name in account_network_map:
        networks = account_network_map[network_name]
        if len(networks) > 1:
            logger.info("Skipped %s because of multiple non-BioPAX matches in the account" % (network_name))

    logger.info("-----------------")
    for network_name in skipped:
        logger.info("Skipped %s" % (network_name))

    return network_id_map

def search_for_non_biopax_networks(ndex):
    search_result = ndex.search_networks(search_string="", account_name=ndex.username, size=10000)
    if "networks" in search_result:
        networks_in_account = search_result["networks"]
    else:
        networks_in_account = search_result

    logger.info("%s networks in account %s" % (len(networks_in_account), ndex.username))

    account_non_biopax_network_map = {}
    biopax_network_count = 0
    for network in networks_in_account:
        name = network["name"]
        skip = False
        if "properties" in network:
            for property in network["properties"]:
                property_name = property["predicateString"]
                if property_name == "sourceFormat" or property_name == "ndex:sourceFormat":
                    if property["value"] == "BIOPAX":
                        skip = True
                        biopax_network_count += 1
                        break
        if not skip:
            if name in account_non_biopax_network_map:
                networks = account_non_biopax_network_map[name]
            else:
                networks = []
                account_non_biopax_network_map[name] = networks
            networks.append(network)

    logger.info("%s BioPAX networks in account %s" % (biopax_network_count, ndex.username))
    for name in account_non_biopax_network_map:
        networks = account_non_biopax_network_map[name]
        if len(networks) > 1:
            logger.info("%s duplicate non-biopax networks for %s" % (len(networks), name))
    logger.info("%s non-BioPAX networks in account %s" % (len(account_non_biopax_network_map), ndex.username))

    return account_non_biopax_network_map

def check_upload_account(username, ndex, dirpath, nci_table):
    search_result = ndex.search_networks(search_string="*",  account_name=username, size=10000)

    if "networks" in search_result:
        networks_in_account = search_result["networks"]
    else:
        networks_in_account = search_result

    account_sif_networks = []
    account_sif_names = []
    for network in networks_in_account:
        if "properties" in network:
            for property in network["properties"]:
                property_name = property["predicateString"]
                if property_name == "sourceFormat" or property_name == "ndex:sourceFormat":
                    if property["value"] != "BIOPAX":
                        account_sif_networks.append(network)
                        account_sif_names.append(network.get("name"))
                        break

    logger.info("%s non-BioPAX networks in account %s" % (len(account_sif_names), username))

    upload_names = []
    for path in listdir(dirpath):
        upload_names.append(network_name_from_path(path))
    logger.info("%s networks to upload / update" % (len(upload_names)))

    # Raise an exception if any of the upload_names are in the account_sif_names multiple times

    nci_names = []
    nci_corrected_names = []

    original_to_corrected_map = {}
    for row in nci_table:
        original_name = None
        if "Pathway Name" in row:
            original_name = row["Pathway Name"]
            nci_names.append(original_name)
        if "Corrected Pathway Name" in row:
            c_name = row["Corrected Pathway Name"]
            if c_name != "":
                nci_corrected_names.append(c_name)
                original_to_corrected_map[original_name] = c_name

    logger.info("there are %s NCI networks specified in the NCI table:" % (len(nci_names)))

    all_nci_names = list(set(nci_corrected_names).union(set(nci_names)))

    # case 1: NCI network names NOT in account_sif names:
    nci_not_account_sifs = list(set(nci_names).difference(set(account_sif_names)))
    logger.info("%s NCI networks out of %s are not in the account:" % (len(nci_not_account_sifs), len(nci_names)))
    for name in nci_not_account_sifs:
        logger.info(name)

    # case 1a: Corrected NCI names NOT in account_sif names:
    nci_corrected_not_account_sifs = list(set(nci_corrected_names).difference(set(account_sif_names)))
    logger.info("%s NCI (corrected name) networks out of %s are not in the account:" % (len(nci_corrected_not_account_sifs), len(nci_corrected_names)))
    for name in nci_corrected_not_account_sifs:
        logger.info(name)

    # case 2: NCI network names NOT in upload names:
    nci_original_not_upload = list(set(nci_names).difference(set(upload_names)))
    logger.info("%s NCI original names out of %s are not in the %s upload names:" % (len(nci_original_not_upload), len(nci_names), len(upload_names)))
    for name in nci_original_not_upload:
        logger.info(name)

    # case 2a: Corrected NCI names NOT in upload names:
    nci_corrected_not_upload = list(set(nci_corrected_names).difference(set(upload_names)))
    logger.info("%s NCI (corrected name) networks out of %s are not in the %s upload names:" % (len(nci_corrected_not_upload), len(nci_corrected_names), len(upload_names)))
    for name in nci_corrected_not_upload:
        logger.info(name)

    # case 4: NCI Networks for which neither the original or corrected name is in the upload list
    # case 2: NCI network names NOT in upload names:
    nci_not_upload = []
    for original_name in nci_names:
        if original_name in upload_names:
            continue
        corrected_name = original_to_corrected_map.get(original_name)
        if not corrected_name:
            nci_not_upload.append([original_name])
            continue
        if corrected_name in upload_names:
            continue
        else:
            nci_not_upload.append([original_name, corrected_name])

    logger.info("%s NCI networks from table not in the %s upload names:" % (len(nci_not_upload), len(upload_names)))
    for item in nci_not_upload:
        if len(item) == 1:
            logger.info(item[0])
        else:
            logger.info("%s -> %s" % (item[0], item[1]))


    # case 5: duplicates of upload names in account_sif names:

    # case 6: upload names not matching any NCI names: CREATE
    upload_not_nci = list(set(upload_names).difference(set(all_nci_names)))
    logger.info("%s upload networks out of %s do not match any of the names from the nci table (corrected or original) and therefore will be created :" % (len(upload_not_nci), len(upload_names)))
    for name in upload_not_nci:
        logger.info(name)

    # case 7: upload names matching some NCI name: UPDATE
    upload_and_nci = list(set(upload_names).intersection(set(all_nci_names)))
    logger.info("%s upload networks out of %s that match an nci name (corrected or original) and therefore will be updated):" % (len(upload_and_nci), len(upload_names)))

    # case 8: account non-BioPAX names not matching any upload name
    account_sif_not_upload = list(set(account_sif_names).difference(set(upload_names)))
    logger.info("%s account (non-BioPAX) networks out of %s do not match any of the upload networks :" % (len(account_sif_not_upload), len(account_sif_names)))
    for name in account_sif_not_upload:
        logger.info(name)

    # case 8: upload names not in account (non-BioPAX)  names=
    upload_not_account_sif = list(set(upload_names).difference(set(account_sif_names)))
    logger.info("%s upload networks out of %s do not match any of the account (non-BioPAX)  networks :" % (len(upload_not_account_sif), len(upload_names)))
    for name in upload_not_account_sif:
        logger.info(name)




def network_name_from_path(path):
    base = basename(path)
    split = splitext(base)
    return split[0]


def remove_my_orphans(network):
    before_node_count = nx.number_of_nodes(network)
    network.remove_orphan_nodes()
    after_node_count = nx.number_of_nodes(network)
    delta = before_node_count - after_node_count
    if delta > 0:
        logger.info("removed " + str(delta) + " orphans")


def _check_filter_(the_filter):
    if the_filter is None:
        return False
    elif the_filter in ["cravat_1", "ndex_1"]:
        return the_filter
    else:
        raise "unknown the_filter: " + str(the_filter)


def _check_layout_(layout):
    if layout is None:
        return False
    elif layout in ["directed_flow"]:
        return layout
    else:
        raise "unknown layout" + str(layout)


def _check_template_network_(template):
    if template is None:
        return False
    elif not isinstance(template, NdexGraph):
        raise "template_network is not an NdexGraph: " + str(template)
    else:
        logger.info("style template network: " + str(template.get_name()))
        return template


CRAVAT_FILTER_LIST = ["neighbor-of", "interacts-with", "controls-state-change-of"]


def cravat_edge_filter(network):
    for edge_id in network.edgemap.keys():
        interaction = str(network.get_edge_attribute_value_by_id(edge_id, 'interaction'))
        if interaction in CRAVAT_FILTER_LIST:
            network.remove_edge_by_id(edge_id)

    remove_my_orphans(network)


# Next:
# get pmids for edges in tuple to edge map
# make removal conditional on NOT removing any unique pmids.
# - a neighbor of edge is preserved if it has a pmid that is not associated with any other edge
# - csc edge is preserved if it has a pmid that is not associated any more specific edge
# hmmm... we can abstract this to finding "more specific edges" for each edge that we
# consider pruning.
NDEX_FILTER_SUBSUMPTION = {
    "neighbor-of": ["controls-state-change-of",
                    # First protein controls a reaction that changes the state of the second protein.
                    "controls-transport-of",
                    # First protein controls a reaction that changes the cellular location of the second protein.
                    "controls-phosphorylation-of"  # First protein controls a reaction that changes the phosphorylation status of the second protein.
                    "controls-expression-of",
                    # First protein controls a conversion or a template reaction that changes expression of the second protein.
                    "catalysis-precedes",
                    # First protein controls a reaction whose output molecule is input to another reaction controled by the second protein.
                    "in-complex-with",  # Proteins are members of the same complex.
                    "interacts-with",  # Proteins are participants of the same MolecularInteraction.
                    "consumption-controled-by",
                    # The small molecule is consumed by a reaction that is controled by a protein
                    "controls-production-of",
                    # The protein controls a reaction of which the small molecule is an output.
                    "controls-transport-of-chemical"  # The protein controls a reaction that changes cellular location of the small molecule.
                    "chemical-affects",  # A small molecule has an effect on the protein state.
                    "reacts-with",  # Small molecules are input to a biochemical reaction
                    "used-to-produce"  # A reaction consumes a small molecule to produce another small molecule.
                    ],
    "controls-state-change-of": [
        "controls-transport-of",
        "controls-phosphorylation-of",
        "controls-expression-of"
    ]
}


def ndex_edge_filter(network):
    map = create_tuple_to_edge_map(network)
    remove_subsumed_edges_of_type_in_network("neighbor-of", map, network)
    remove_subsumed_edges_of_type_in_network("controls-state-change-of", map, network)
    # remove_my_orphans(network)
    return True

    # for tuple_key, edges in map.items():
    #     # get the neighbor-of edges
    #     neighbor_of_edge_ids = []
    #     for edge in edges:
    #         if edge["interaction"] == "neighbor-of":
    #             neighbor_of_edge_ids.append = edge["edge_id"]
    #
    #     n_neighbors = len(neighbor_of_edge_ids)
    #
    #     # remove unwanted neighbor-of edges, if any
    #     if n_neighbors < len(edges) and n_neighbors > 0:
    #         if n_neighbors == 1:
    #             # one neighbor-of is the only edge, keep it
    #             continue
    #         else:
    #             # remove all but one
    #             for edge_id in range(1, n_neighbors - 1):
    #                 network.remove_edge_by_id(edge_id)
    #                 logger.info("removing edge " + str(edge_id) + " " + "neighbor-of")
    #             continue
    #
    #     # controls-state-change-of
    #     # if there is both a controls-state-change-of edge and
    #     # controls-phosphorylation-of edge from A to B, then
    #     # remove the controls-phosphorylation-of edge
    #     node_ids = tuple_key.split("_")
    #     node_a = node_ids[0]
    #     node_b = node_ids[1]
    #     csc_a_b = None
    #     csc_b_a = None
    #     cp_a_b = None
    #     cp_b_a = None
    #     for edge in edges:
    #         if edge["interaction"] == "controls-state-change-of":
    #             if edge["source_id"] == node_a:
    #                 csc_a_b = edge["edge_id"]
    #             else:
    #                 csc_b_a = edge["edge_id"]
    #
    #         if edge["interaction"] == "controls-phosphorylation-of":
    #             if edge["source_id"] == node_a:
    #                 cp_a_b = edge["edge_id"]
    #             else:
    #                 cp_b_a = edge["edge_id"]
    #     if csc_a_b and cp_a_b:
    #         network.remove_edge_by_id(csc_a_b)
    #         logger.info("removing edge " + str(csc_a_b) + " " + "controls-state-change-of")
    #     if csc_b_a and cp_b_a:
    #         network.remove_edge_by_id(csc_b_a)
    #         logger.info("removing edge " + str(csc_b_a) + " " + "controls-state-change-of")


def remove_subsumed_edges_of_type_in_network(edge_type, tuple_to_edge_map, network):
    for tuple_key, edges in tuple_to_edge_map.items():
        node_ids = tuple_key.split("_")
        node_a_id = node_ids[0]
        forward_edges = []
        backward_edges = []
        for edge in edges:
            if edge["source_id"] == node_a_id:
                forward_edges.append(edge)
            else:
                backward_edges.append(edge)

        remove_subsumed_edges_of_type(edge_type, forward_edges, network)
        remove_subsumed_edges_of_type(edge_type, backward_edges, network)


def remove_subsumed_edges_of_type(edge_type, edges, network):
    if len(edges) == 0:
        return False

    subsuming_interactions = NDEX_FILTER_SUBSUMPTION.get(edge_type)
    if not subsuming_interactions:
        # this edge type is never subsumed
        return False

    subsuming_edges = []
    subsuming_pmids = []
    edges_to_be_subsumed = []
    for edge in edges:
        if edge["interaction"] == edge_type:
            edges_to_be_subsumed.append(edge)
        if edge["interaction"] in subsuming_interactions:
            subsuming_edges.append(edge)
            if "pmid" in edge:
                for pmid in edge["pmid"]:
                    subsuming_pmids.append(pmid)

    if len(subsuming_edges) == 0 or len(edges_to_be_subsumed) == 0:
        # nobody to be subsumed or nobody to do the subsuming :-)
        return False

    some_edge_removed = False
    for edge in edges_to_be_subsumed:
        if "pmid" in edge:
            # check to be sure that all pmids cited
            # by the "to be subsumed" edges are cited
            # in some subsuming edge to ensure that
            # removal of the edge does not lose information
            for pmid in edge["pmid"]:
                if pmid in subsuming_pmids:
                    network.remove_edge_by_id(edge["edge_id"])
                    logger.info("removing edge " + str(edge["edge_id"]) + " : " + edge_type)
                    some_edge_removed = True
        else:
            # there is no pmid in the subsumed edge,
            # it can therefore be removed without loss of information
            network.remove_edge_by_id(edge["edge_id"])
            logger.info("removing edge " + str(edge["edge_id"]) + " : " + edge_type)
            some_edge_removed = True

    return some_edge_removed


def create_tuple_to_edge_map(network):
    map = {}
    for edge_id in network.edgemap:
        s, t = network.edgemap[edge_id]
        interaction = network.get_edge_attribute_value_by_id(edge_id, "interaction")
        if s > t:
            tuple_key = str(s) + "_" + str(t)
        else:
            tuple_key = str(t) + "_" + str(s)
        if not tuple_key in map:
            map[tuple_key] = [{"edge_id": edge_id, "interaction": interaction, "source_id": s, "target_id": t}]
        else:
            map[tuple_key].append({"edge_id": edge_id, "interaction": interaction, "source_id": s, "target_id": t})
    return map


def load_nci_table_to_dicts(path):
    table = []
    with open(path, 'rU') as f:
        reader = csv.DictReader(f, dialect='excel-tab')
        for row in reader:
            table.append(row)
    return table


def add_nci_table_properties(G, network_name, nci_table, not_in_nci_table):
    # find the entry for network_name in nci_table
    network_dict = None
    for row in nci_table:
        if "Pathway Name" in row:
            if network_name == row["Pathway Name"]:
                # simple match!
                network_dict = row
                break
        if "Corrected Pathway Name" in row:
            if network_name == row["Corrected Pathway Name"]:
                network_dict = row
                break
    if not network_dict:
        not_in_nci_table.append(network_name)
        return

    if "PID" in network_dict:
        pid = network_dict["PID"]
        G.set_network_attribute("labels", pid)

    if "Reviewed By" in network_dict:
        reviewed_by = network_dict["Reviewed By"]
        # names = reviewed_by.split(",")
        # reviewers = []
        # for name in names:
        #     reviewers.append(name.strip())
        # G.set_network_attribute("reviewer", reviewers)
        G.set_network_attribute("reviewers", reviewed_by)

    if "Curated By" in network_dict:
        curated_by = network_dict["Curated By"]
        # names = curated_by.split(",")
        # authors = []
        # for name in names:
        #     authors.append(name.strip())
        # G.set_network_attribute("author", authors)
        G.set_network_attribute("author", curated_by)

    # if "Revision Date" in network_dict:
    #     revision_date = network_dict["Revision Date"]
    #     G.set_network_attribute("revised", revision_date)


def load_ebs_file_to_dict(path):
    edge_table = []
    node_table = []
    ebs = {"edge_table": edge_table, "node_table": node_table}

    with open(path, 'rU') as f:
        lines = f.readlines()
        mode = "edge"
        edge_lines = []
        node_lines = []
        edge_fields = []
        node_fields = []
        for index in range(len(lines)):
            line = lines[index]
            if index is 0:
                edge_fields = [h.strip() for h in line.split('\t')]
            elif line == '\n':
                mode = "node_header"
            elif mode is "node_header":
                node_fields = [h.strip() for h in line.split('\t')]
                mode = "node"
            elif mode is "node":
                node_lines.append(line)
            elif mode is "edge":
                edge_lines.append(line)

        edge_reader = csv.DictReader(edge_lines, fieldnames=edge_fields, dialect='excel-tab')
        for dict in edge_reader:
            edge_table.append(dict)

        node_reader = csv.DictReader(node_lines, fieldnames=node_fields, dialect='excel-tab')
        for dict in node_reader:
            node_table.append(dict)

    return ebs

def _get_node_type(ebs_type):
    if ebs_type is None:
        return "Other"
    if ebs_type in ["SmallMolecule", "SmallMoleculeReference"]:
        return "SmallMolecule"
    if ebs_type in ["Complex", "ComplexAssembly"]:
        return "Complex"
    if ebs_type in ["Protein", "ProteinReference"]:
        return "Protein"
    if ebs_type in ["Rna", "RnaReference"]:
        return "Rna"
    return "Other"

import requests

def ebs_to_network(ebs, name="not named"):
    G = NdexGraph()
    G.set_name(name)
    node_id_map = {}
    identifier_citation_id_map = {}
    node_mapping = {}
    mg = mygene.MyGeneInfo()

    # Create Nodes
    # PARTICIPANT	PARTICIPANT_TYPE	PARTICIPANT_NAME	UNIFICATION_XREF	RELATIONSHIP_XREF

    id_list = []
    for node in ebs.get("node_table"):
        if "PARTICIPANT" in node:
            participant = node["PARTICIPANT"]
            participant = clean_brackets(participant)
            if participant is not None and '_HUMAN' in participant and node_mapping.get(participant) is None:
                id_list.append(participant)

    url = 'https://biodbnet-abcc.ncifcrf.gov/webServices/rest.php/biodbnetRestApi.json?method=db2db&input=uniprot entry name&inputValues=' + ','.join(id_list) + '&outputs=genesymbol&taxonId=9606&format=row'
    look_up_req = requests.get(url)
    look_up_json = look_up_req.json()
    if look_up_json is not None:
        for bio_db_item in look_up_json:
            gene_symbol_mapping[bio_db_item.get('InputValue')] = bio_db_item.get('Gene Symbol')
            node_mapping[bio_db_item.get('InputValue')] = bio_db_item.get('Gene Symbol')

    for node in ebs.get("node_table"):
        attributes = {}
        if "PARTICIPANT" in node:
            participant = node["PARTICIPANT"]
            participant = clean_brackets(participant)
            participant_symbol = ''
            if participant is not None:
                #participant = participant.replace('_HUMAN', '')

                participant_symbol = gene_symbol_mapping.get(participant)
                #=====================================
                # if the node is not a protien type
                # then skip mygene.info look up
                #=====================================
                if node.get('PARTICIPANT_TYPE') is not None and node.get('PARTICIPANT_TYPE') != 'ProteinReference':
                    participant_symbol = participant

                if participant_symbol is None:
                    participant_symbol = mg.query(participant, species='human')
                    if participant_symbol is not None and participant_symbol.get('hits') is not None and len(participant_symbol.get('hits')) > 0:
                        ph = participant_symbol.get('hits')[0]
                        if 'symbol' in ph:
                            participant_symbol = str(ph.get('symbol'))
                            gene_symbol_mapping[participant] = participant_symbol
                    else:
                        participant_symbol = None

            participant_name = None
            if participant_symbol is not None:
                node_name = participant_symbol
            else:
                node_name = participant

            node_name_tmp = participant

            if "PARTICIPANT_NAME" in node:
                name = node["PARTICIPANT_NAME"]
                if name is not None and len(name) > 0:
                    attributes["name"] = name
                    participant_name = name

            node_type = _get_node_type(node.get("PARTICIPANT_TYPE"))

            if "UNIFICATION_XREF" in node:
                alias_string = node["UNIFICATION_XREF"]
                if alias_string is not None and alias_string is not "":
                    aliases = alias_string.split(";")
                    attributes["alias"] = aliases
                    # attributes["represents"] = alias[0] - can't take first alias for ndexebs.
                    # Need to resolve uniprot primary id for the gene
                    if node_type == "Other":
                        node_type = layouts.aliases_to_node_type(aliases)

            attributes["type"] = node_type

            if node_name_tmp.startswith("CHEBI") and participant_name:
                node_name = participant_name

            # check mygene.info

            #if there is a result use the gene symbol and add the uniprot id as an alias

            node_id = G.add_new_node(node_name, attributes)
            node_id_map[participant] = node_id

    # Create Edges
    # PARTICIPANT_A	INTERACTION_TYPE	PARTICIPANT_B	INTERACTION_DATA_SOURCE
    # INTERACTION_PUBMED_ID	PATHWAY_NAMES	MEDIATOR_IDS
    for edge in ebs.get("edge_table"):
        if "INTERACTION_TYPE" not in edge:
            raise "No interaction type for edge " + str(edge)
        if "PARTICIPANT_A" not in edge:
            raise "No participant A in edge " + str(edge)
        if "PARTICIPANT_B" not in edge:
            raise "No participant B in edge " + str(edge)

        attributes = {}
        interaction = edge["INTERACTION_TYPE"]
        attributes["interaction"] = interaction

        if interaction in DIRECTED_INTERACTIONS:
            attributes["directed"] = True

        # handle pmids as edge attribute
        if "INTERACTION_PUBMED_ID" in edge:
            pmid_string = edge["INTERACTION_PUBMED_ID"]
            if pmid_string is not None and pmid_string is not "":
                pmids = pmid_string.split(";")
                attributes["pmid"] = pmids

        participant_a = clean_brackets(edge["PARTICIPANT_A"])
        participant_b = clean_brackets(edge["PARTICIPANT_B"])
        #if participant_a is not None:
        #    participant_a = participant_a.replace('_HUMAN', '')
        #if participant_b is not None:
        #    participant_b = participant_b.replace('_HUMAN', '')
        source_node_id = node_id_map.get(participant_a)
        target_node_id = node_id_map.get(participant_b)
        edge_id = G.add_edge_between(source_node_id, target_node_id, interaction=interaction, attr_dict=attributes)

        # handle pmids in CX citation and edgeCitation aspect representations in networkn network object
        if "pmid" in attributes:
            prefixed_pmids = []
            for pmid in attributes["pmid"]:
                prefixed_pmids.append("pmid:" + str(pmid))

            for prefixed_pmid in prefixed_pmids:
                if prefixed_pmid not in identifier_citation_id_map:
                    citation_id = G.add_citation(identifier=prefixed_pmid)
                    identifier_citation_id_map[prefixed_pmid] = citation_id

                    G.add_citation_to_edge(edge_id, prefixed_pmid) #add_edge_citation_ref(edge_id, citation_id)
                else:
                    citation_id = identifier_citation_id_map[prefixed_pmid]
                    G.add_citation_to_edge(edge_id, prefixed_pmid) #add_edge_citation_ref(edge_id, citation_id)

    # save gene_symbol_mapping to file (caching)
    with open(join(current_directory, 'gene_symbol_mapping.json'), 'w') as outfile:
        json.dump(gene_symbol_mapping, outfile, indent=4)

    return G

#def get_mygene_info_label():

NCI_DESCRIPTION_TEMPLATE = ("<i>%s</i> was derived from the latest BioPAX3 version of the Pathway Interaction Database (PID)"
                                " curated by NCI/Nature. The BioPAX was first converted to Extended Binary SIF (EBS) by"
                                " the PAXTools v5 utility. It was then processed to remove redundant"
                                " edges, to add a 'directed flow' layout, and to add a graphic style using Cytoscape Visual Properties."
                                " This network can be found in searches using its original PID accession id, present in"
                                " the 'labels' property.")

def clean_brackets(clean_this_string):
    return_this = ''
    if clean_this_string is not None:
        return_this = clean_this_string.replace('[','').replace(']','')
        #if ',' in return_this:
        #    return_this = return_this.split(',')[0].upper()
        return return_this
    else:
        return clean_this_string


def get_my_gene_info(gene_id):
    alt_term_id = []
    if(len(gene_id) > 0):
        r_json = {}
        try:
            url = 'http://mygene.info/v3/query?q=' + gene_id

            r = requests.get(url)
            r_json = r.json()
            if 'hits' in r_json and len(r_json['hits']) > 0:
                for alt_term in r_json['hits']:
                    if(isinstance(alt_term['symbol'], list)):
                        alt_term_id.append(alt_term['symbol'][0].upper())
                    else:
                        alt_term_id.append(alt_term['symbol'].upper())

                #gene_symbol = r_json['hits'][0]['symbol'].upper()
                return alt_term_id
        except Exception as e:
            logger.exception('Caught exception')
            return {'hits': [{'symbol': gene_id, 'entrezgene': '', 'name': 'Entrez results: 0'}]}

        return ["UNKNOWN"]
    else :
        return ["UNKNOWN"]





from os.path import isfile, expanduser
import ndex2
import json
nicecx = ndex2.NiceCXNetwork
Ndex2 = ndex2.Ndex2



example_config = {
    "connections" : {
        "main" : {
            "server" : "http://dev.ndexbio.org",
            "username" : "my_user",
            "password" : "my_pass"
        }
    }
}

def load_tutorial_config(connection_name):
    username = None
    password = None
    config_file = expanduser("~/ndex_tutorial_config.json")
    server = "http://public.ndexbio.org"

    if isfile(config_file):
        file = open(config_file, "r")
        config = json.load(file)
        file.close()
        connections = config.get("connections")
        if connections:
            if connection_name:
                connection = connections.get(connection_name)
                if connection and connection.get("password") and config.get("username"):
                    if connection.get("server"):
                        server = connection.get("server")
                    username = connection.get("username")
                    password = connection.get("password")
            else:
                print("Error: " + str(connection_name) + " does not define both username and password")
        else:
            print("Error: " + config_file + " does not define any connections")
    else:
        print("Error: " + config_file + " was not found")

    return server, username, password



# -*- coding: utf-8 -*-

import os
import configparser
import logging
from ndexutil.exceptions import ConfigError


class NDExUtilConfig(object):
    """
    Instances provide configuration information
    stored for ndexutil package in user's home directory
    """
    CONFIG_FILE = '.ndexutils.conf'

    USER = 'user'
    PASSWORD = 'password'
    SERVER = 'server'

    def __init__(self, conf_file=None):
        """Constructor
        """
        self._conf_file = conf_file
        self._homedir = os.path.expanduser('~')

    def set_home_directory(self, path):
        """Sets alternate home directory
        :param path: Alternate home directory path
        """
        if path is not None and '~' in path:
            self._homedir = os.path.expanduser(path)
        else:
            self._homedir = path

    def get_home_directory(self):
        """
        Returns home directory path
        :returns: Path to home directory
        """
        return self._homedir

    def get_config_file(self):
        """
        Gets config file
        :return:
        """
        if self._conf_file is None:
            return os.path.join(self._homedir, NDExUtilConfig.CONFIG_FILE)
        return self._conf_file

    def get_config(self):
        """
        Gets configparser object loaded with data from
        :return:
        """
        if not os.path.isfile(self.get_config_file()):
            raise ConfigError('No configuration file found')

        parser = configparser.ConfigParser()
        parser.read(self.get_config_file())
        return parser


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `streamtsvloader` module."""

import tempfile
import shutil
import os
import io
import unittest
from ndexutil.tsv.streamtsvloader import StreamTSVLoader
from ndexutil.tsv.streamtsvloader import CXStreamWriter
from ndexutil.exceptions import NDExUtilError
import ndex2


class TeststreamTSVLoader(unittest.TestCase):
    """
    Tests streamtsvloader.py
    """
    def setUp(self):
        """Set up test fixtures, if any."""
        pass

    def tearDown(self):
        """Tear down test fixtures, if any."""
        pass

    def test_write_pre_metadata_none_as_stream(self):
        writer = CXStreamWriter(None)
        try:
            writer.write_pre_metadata({})
            self.fail('Expected NDExUtilError')
        except NDExUtilError as ne:
            self.assertEqual('Output stream is None', str(ne))

    def test_write_aspect_fragment_called_before_write_premeta(self):
        writer = CXStreamWriter(None)
        try:
            writer.write_aspect_fragment({})
            self.fail('Expected NDExUtilError')
        except NDExUtilError as ne:
            self.assertTrue('Data aspects can only' in str(ne))

    def test_write_post_metadata_called_before_write_premeta(self):
        writer = CXStreamWriter(None)
        try:
            writer.write_post_metadata({})
            self.fail('Expected NDExUtilError')
        except NDExUtilError as ne:
            self.assertTrue('Post metadata aspect can only' in str(ne))

    def test_write_pre_metadata_twice(self):
        stream = io.StringIO()
        writer = CXStreamWriter(stream)
        writer.write_pre_metadata({})
        try:
            writer.write_pre_metadata({})
            self.fail('Expected NDExUtilError')
        except NDExUtilError as ne:
            self.assertTrue('PreMetadata has already' in str(ne))

    def test_write_post_metadata_twice(self):
        stream = io.StringIO()
        writer = CXStreamWriter(stream)
        writer.write_pre_metadata({})
        writer.write_post_metadata({})
        try:
            writer.write_post_metadata({})
            self.fail('Expected NDExUtilError')
        except NDExUtilError as ne:
            self.assertTrue('Post metadata aspect can only' in str(ne))

    def test_creating_network(self):
        temp_dir = tempfile.mkdtemp()
        try:
            here = os.path.dirname(__file__)
            tsvfile = os.path.join(here, 'ctd_test.tsv')
            with open (tsvfile, 'r') as tsvfile:
                tmpcx = 'out.cx'
                with open(tmpcx, "w") as out:
                    nicecx = ndex2.create_nice_cx_from_file(os.path.join(here, 'gene-disease-style.cx'))
                    loader = StreamTSVLoader(os.path.join(here, 'ctd-gene-disease-2019-norm-plan-collapsed.json'),nicecx)
                    loader.write_cx_network(tsvfile, out, [{'n': 'name', 'v': "CTD: gene-disease association (Human)"},
                                                           {'n': 'version', 'v': "0.0.1"}], batchsize=4)

                nicecx = ndex2.create_nice_cx_from_file(tmpcx)
                self.assertEqual(len(nicecx.networkAttributes), 3)
                self.assertEqual(len(nicecx.edges), 49)
                self.assertEqual(len(nicecx.nodes), 50)
                node_attr_cnt = 0
                for key, value in nicecx.nodeAttributes.items():
                    node_attr_cnt += len(value)
                self.assertEqual(node_attr_cnt, 50)
                edge_attr_cnt = 0
                for key, value in nicecx.edgeAttributes.items():
                    edge_attr_cnt += len(value)
                self.assertEqual(edge_attr_cnt, 147)
                self.assertEqual(len(nicecx.opaqueAspects.get("cyVisualProperties")), 3)
        finally:
            shutil.rmtree(temp_dir)

    def test_network_without_represent(self):
        temp_dir = tempfile.mkdtemp()
        try:
            here = os.path.dirname(__file__)
            tsvfile = os.path.join(here, 'BRCA-2012-TP53-pathway.txt_with_a_b.csv')
            with open (tsvfile, 'r') as tsvfile:
                #with open (os.path.join ( temp_dir, "out.cx"),"w") as out:
                tmpcx = 'out.cx'
                with open(tmpcx, "w") as out:
                    loader = StreamTSVLoader(os.path.join(here, 'BRCA-2012-loadplan.json'), None)
                    loader.write_cx_network(tsvfile, out, [{'n': 'name', 'v': "test pathway"},
                                                           {'n':'version', 'v': "0.0.1"}])

                nicecx = ndex2.create_nice_cx_from_file(tmpcx)
                self.assertEqual(len(nicecx.networkAttributes), 3)
                self.assertEqual(len(nicecx.edges), 8)
                self.assertEqual(len(nicecx.nodes), 9)
                self.assertEqual(nicecx.nodes[0].get('r'), None)
                node_attr_cnt = 0
                for key, value in nicecx.nodeAttributes.items():
                    node_attr_cnt += len(value)
                self.assertEqual(node_attr_cnt, 25)

        finally:
            shutil.rmtree(temp_dir)



 #   def test_parse_arguments(self):
 #           here = os.path.dirname(__file__)
 #           tsvfile = os.path.join("/Users/chenjing/git/ndexctdloader/ndexctdloader", 'collapsed_Homo_sapiens.tsv')
 #           with open(tsvfile, 'r') as tsvfile:
                # with open (os.path.join ( temp_dir, "out.cx"),"w") as out:
 #               with open("/Users/chenjing/Downloads/ctdout.cx", "w") as out:
 #                   nicecx = ndex2.create_nice_cx_from_file(os.path.join(here, 'gene-disease-style.cx'))
 #                   loader = StreamTSVLoader(os.path.join(here, 'ctd-gene-disease-2019-norm-plan-collapsed.json'),
 #                                            nicecx)
 #                   loader.write_cx_network(tsvfile, out, [{'n': 'name', 'v': "CTD: gene-disease association (Homo_Sapiens)"},
 #                                                          {'n': 'version', 'v': "0.0.1"}], batchsize=5000)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `tsv2nicecx2` module."""

import tempfile
import shutil
import os
import io
import unittest
import pandas as pd
import json
from ndexutil.tsv import tsv2nicecx2
from ndexutil.exceptions import NDExUtilError
import ndex2


class Testtsv2nicecx2(unittest.TestCase):
    """
    Tests tsv2nicecx2.py
    """
    def setUp(self):
        """Set up test fixtures, if any."""
        pass

    def tearDown(self):
        """Tear down test fixtures, if any."""
        pass

    def test_context_set_correctly_on_network(self):
        temp_dir = tempfile.mkdtemp()
        try:
            here = os.path.dirname(__file__)
            tsvfile = os.path.join(here, 'ctd_test.tsv')
            loadplanfile = os.path.join(here,
                                        'ctd-gene-disease-2019-norm-'
                                        'plan-collapsed.json')

            with open(loadplanfile, 'r') as f:
                loadplan = json.load(f)

            df = pd.read_csv(tsvfile, sep='\t')
            first = tsv2nicecx2.convert_pandas_to_nice_cx_with_load_plan(df, loadplan,
                                                                         name='mynetwork',
                                                                         description='mydesc')

            nadd = [{'n': 'hi', 'v': 'data'}]
            second = tsv2nicecx2.convert_pandas_to_nice_cx_with_load_plan(df, loadplan,
                                                                          network_attributes=nadd)
            for net in [first, second]:
                res = net.get_network_attribute('@context')
                self.assertEqual('{"ncbigene": "http://ctdbase.org/detail.go'
                                 '?type=gene&acc=", "OMIM": "http://ctdbase.'
                                 'org/detail.go?type=disease&acc=OMIM:", "ME'
                                 'SH": "http://ctdbase.org/detail.go?type=dis'
                                 'ease&acc=MESH:", "pubmed": "http://ctdbase.'
                                 'org/detail.go?type=reference&acc="}', res['v'])
                self.assertEqual(len(net.edges), 49)
                self.assertEqual(len(net.nodes), 50)
                node_attr_cnt = 0
                for key, value in net.nodeAttributes.items():
                    node_attr_cnt += len(value)
                self.assertEqual(node_attr_cnt, 50)
                edge_attr_cnt = 0
                for key, value in net.edgeAttributes.items():
                    edge_attr_cnt += len(value)
                self.assertEqual(edge_attr_cnt, 147)

            self.assertEqual(len(first.networkAttributes), 3)
            self.assertEqual(len(second.networkAttributes), 2)
            self.assertEqual('data', second.get_network_attribute('hi')['v'])

        finally:
            shutil.rmtree(temp_dir)




import json
import os
from os import path
import jsonschema
import pandas as pd
import time
import re
import logging
from ndex2cx.nice_cx_builder import NiceCXBuilder
from ndex2.client import Ndex2

version="0.1"

node_id_lookup = {}

logger = logging.getLogger(__name__)


class ContentImporter(object):
    def __init__(self, server, username, password, **attr):
        self.update_mapping = {}
        self.server = server
        self.username = username
        self.password = password
        self.network = None
        self.ndex = Ndex2(self.server, self.username, self.password)

        networks = self.ndex.get_network_summaries_for_user(self.username)
        for nk in networks:
            if nk.get('name') is not None:
                self.update_mapping[nk.get('name').upper()] = nk.get('externalId')

    def process_file(self, file_name, load_plan_path, name, style_template=None, custom_header=None, delimiter='\t'):
        # ==============================
        # LOAD TSV FILE INTO DATAFRAME
        # ==============================
        if not os.path.isfile(file_name): # If file is not in main directory try the ./data directory
            file_name = os.path.join('data', file_name)

        if not os.path.isfile(load_plan_path): # If file is not in main directory try the ./data directory
            load_plan_path = os.path.join('data', load_plan_path)

        with open(file_name, 'r', encoding='utf-8', errors='ignore') as tsvfile:
            if custom_header is None:
                header = [h.strip() for h in tsvfile.readline().split(delimiter)]

                df = pd.read_csv(tsvfile, delimiter=delimiter, na_filter=False, engine='python', names=header,
                                 dtype=str, error_bad_lines=False, comment='#')
            else:
                if isinstance(custom_header, list):
                    df = pd.read_csv(tsvfile, delimiter=delimiter, na_filter=False, engine='python', names=custom_header,
                                     dtype=str, error_bad_lines=False, comment='#')
                else:
                    raise Exception('Custom header provided was not of type list')

        # =====================
        # LOAD TSV LOAD PLAN
        # =====================
        if load_plan_path is not None:
            try:
                with open(load_plan_path, 'r') as lp:
                    load_plan = json.load(lp)
            except jsonschema.ValidationError as e1:
                logger.exception("Failed to parse the loading plan: " + e1.message)
                logger.error('at path: ' + str(e1.absolute_path))
                logger.error("in block: ")
                logger(e1.instance)
        else:
            raise Exception('Please provide a load plan')

        # ====================
        # UPPERCASE COLUMNS
        # ====================
        rename = {}
        for column_name in df.columns:
            rename[column_name] = column_name.upper()

        network = convert_pandas_to_nice_cx_with_load_plan(df, load_plan)
        network.set_name(name)
        if style_template is not None:
            logger.debug('Applying style from network: ' + style_template)
            network.apply_template(username=self.username, password=self.password, server=self.server,
                               uuid=style_template)

        self.network = network

    def upload_network(self, re_use_metadata=True):
        network_update_key = self.update_mapping.get(self.network.get_name().upper())
        if network_update_key is not None and re_use_metadata in ['true', 'True', 'yes', True]:
            logger.debug("Updating")
            self.update_network_properties(network_update_key)
            message = self.network.update_to(network_update_key, self.server, self.username, self.password)
        else:
            logger.debug("New network")
            message = self.network.upload_to(self.server, self.username, self.password)

        logger.info(message)

    def get_network_properties(self, uuid):

        network_properties_stream = self.ndex.get_network_aspect_as_cx_stream(uuid, 'networkAttributes')

        network_properties = network_properties_stream.json()
        return_properties = {}
        for net_prop in network_properties:
            return_properties[net_prop.get('n')] = net_prop.get('v')

        return return_properties

    def update_network_properties(self, uuid):
        network_properties = self.get_network_properties(uuid)

        for k, v in network_properties.items():
            self.network.set_network_attribute(k, v)

    def print_summary(self):
        print('Loading...')


#=====================
# NON-CLASS FUNCTIONS
#=====================
def convert_pandas_to_nice_cx_with_load_plan(pandas_dataframe, load_plan, max_rows=None,
                                            name=None, description=None,
                                            network_attributes=None, provenance=None):

    # open the schema first
    here = path.abspath(path.dirname(__file__))
    with open(path.join(here, 'loading_plan_schema.json')) as json_file:
        plan_schema = json.load(json_file)

    jsonschema.validate(load_plan, plan_schema)

    node_lookup = {}
    nice_cx_builder = NiceCXBuilder()
    row_count = 0
    t1 = int(time.time()*1000)

    #Add context if they are defined
    context = load_plan.get('context')
    if context:
        if network_attributes is None:
            network_attributes = []
        network_attributes.append({"n": "@context", "v": json.dumps(context)})

    total_row_count = pandas_dataframe.shape
    if len(total_row_count) > 1:
        total_row_count = str(total_row_count[0])
    for index, row in pandas_dataframe.iterrows():
        # As each row is processed, self.G_nx is updated
        process_row(nice_cx_builder, load_plan, row, node_lookup)
        row_count = row_count + 1
        if max_rows and row_count > max_rows + 2:
            break

        if row_count % 2500 == 0:
            logger.info('processing %s out of %s edges' % (str(row_count), total_row_count))

    if network_attributes:
        for attribute in network_attributes:
            if attribute.get("n") == "name":
                nice_cx_builder.set_name(attribute.get("v"))
            else:
                nice_cx_builder.add_network_attribute(name=attribute.get('n'), values=attribute.get('v'),
                                                      type=attribute.get('d'))

    tsv_data_event = {
            "inputs": None,
            "startedAtTime": t1,
            "endedAtTime": int(time.time()*1000),
            "eventType": "TSV network generation",
            "properties": [{
                            "name": "TSV loader version",
                            "value": version
                        }]
        }

    # name and description take precedence over any prior values
    if name:
        nice_cx_builder.set_name(name)
    if description:
        nice_cx_builder.add_network_attribute(name='description', values=description)

    return nice_cx_builder.get_nice_cx()


#==================================
# Process Row USING NiceCX
# Added by Aaron G
#==================================
def process_row(nice_cx_builder, load_plan, row, node_lookup):
    # For each row, we create an edge + edge properties
    # For that edge, we may create elements if they are new
    # - source node + properties
    # - target node + properties
    # - predicate term
    skipped_edge = False

    logger.debug(row)
    source_node = create_node(row, load_plan.get('source_plan'), nice_cx_builder, node_lookup)
    target_node = create_node(row, load_plan.get('target_plan'), nice_cx_builder, node_lookup)

    if source_node is not None and target_node is not None:
        create_edge(nice_cx_builder, source_node, target_node, row, load_plan)


def create_node(row, node_plan, nice_cx_builder, node_lookup):
    use_name_as_id = False
    node_name_column = node_plan['node_name_column']
    node_name_type = None
    if '::' in node_name_column:
        node_name_split = node_name_column.split('::')
        node_plan['node_name_column'] = node_name_split[0]
        node_name_type = node_name_split[1]

    #=======================================
    # IF NO REP_COLUMN USE NODE NAME COLUMN
    #=======================================
    if not node_plan.get('rep_column'):
        node_plan['rep_column'] = node_plan['node_name_column']
        use_name_as_id = True

    if use_name_as_id and node_plan.get('rep_prefix'):
        raise RuntimeError("Id column needs to be defined if id_prefix is defined in your query plan.")

    node_name = row[node_plan['node_name_column']]
    ext_id = row[node_plan['rep_column']]

    if ext_id and node_plan.get('rep_prefix'):
        ext_id = node_plan['rep_prefix'] + ":" + str(ext_id)

    #========================================
    # NAME  |  EXT ID  | ACTION
    #----------------------------------------
    # VALID |  VALID   | NORMAL
    # VALID |  None    | SUB NAME FOR EXT ID
    # None  |  VALID   | SUB EXT ID FOR NAME
    # None  |  None    | SKIP ROW
    #========================================
    if node_name and not ext_id:
        ext_id = node_name
    elif not node_name and ext_id:
        node_name = ext_id
    elif not node_name and not ext_id:
        print('No node name or ext id.  Skipping this node (%s)' % node_plan['node_name_column'])
        return None

    node_id = nice_cx_builder.add_node(name=node_name, represents=ext_id, data_type=node_name_type)

    add_node_attributes(nice_cx_builder, node_id, node_plan, row)

    return node_id


def create_edge(nice_cx_builder, src_node_id, tgt_node_id, row, import_plan):
    predicate_str = None
    edge_plan = import_plan.get('edge_plan')
    if edge_plan.get('predicate_id_column'):
        predicate_str = row[edge_plan['predicate_id_column']]

    if not predicate_str and edge_plan.get('default_predicate'):
        predicate_str = edge_plan['default_predicate']

    if not predicate_str:
        raise RuntimeError("Value for predicate string is not found in this row.")
    if edge_plan.get("predicate_prefix"):
        predicate_str = edge_plan['predicate_prefix'] + ":" + predicate_str

    edge_id = nice_cx_builder.add_edge(source=src_node_id, target=tgt_node_id, interaction=predicate_str)

    add_edge_attributes(nice_cx_builder, edge_id, edge_plan, row)

    # Deal with citiations
    citation_id = None
    if edge_plan.get("citation_id_column"):
        citation_id = str(row[edge_plan['citation_id_column']])

    if citation_id is not None:
        citation_id = citation_id.replace(';', ',')
        citation_id = citation_id.replace('|', ',')
        if edge_plan.get("citation_id_prefix"):
            citation_id_temp = re.split('\s*,\s*', citation_id)
            citation_id = []
            for c in citation_id_temp:
                citation_id.append(edge_plan.get("citation_id_prefix") + ':' + c)
            #citation_id = edge_plan.get("citation_id_prefix") + ':' + citation_id
        else:
            citation_id = re.split('\s*,\s*', citation_id)

        nice_cx_builder.add_edge_attribute(property_of=edge_id, name='citation', values=citation_id, type='list_of_string')

    #if citation_id and edge_plan.get("citation_id_prefix"):
    #    newList = []
    #    for cid in citation_id:
    #        newList.append(edge_plan["citation_id_prefix"] + ":" + cid)
    #    citation_id = newList

    #if citation_id:
    #    nice_cx.set_edge_attribute(edge_id, 'citation', citation_id, type='list_of_string')
    #    #nice_cx.set_edge_attribute(edge_id, 'citation_ids', citation_id, type='list_of_string')


valid_cx_data_types = ['boolean', 'byte', 'char', 'double', 'float', 'integer', 'long', 'short', 'string', 'list_of_boolean',
 'list_of_byte', 'list_of_char', 'list_of_double', 'list_of_float', 'list_of_integer', 'list_of_long',
 'list_of_short','list_of_string']


def add_node_attributes(nice_cx_builder, node_element, load_plan, row):
    if load_plan.get('property_columns'):
        for column_raw_temp in load_plan['property_columns']:
            if isinstance(column_raw_temp, dict):
                column_raw = column_raw_temp
            else:
                if '::' in column_raw_temp:
                    column_split = column_raw_temp.split('::')
                    if len(column_split) > 1:
                        column_raw = {
                            'column_name': column_split[0],
                            'attribute_name': column_split[0],
                            'data_type': column_split[1]
                        }
                    else:
                        column_raw = {
                            'column_name': column_raw_temp,
                            'attribute_name': column_raw_temp
                        }
                else:
                    column_raw = {
                        'column_name': column_raw_temp,
                        'attribute_name': column_raw_temp
                    }

            type_temp = column_raw.get('data_type')
            value = None

            if column_raw.get('column_name'):
                value = row.get(column_raw['column_name'])

            if (value is None) and column_raw.get('default_value'):
                value = column_raw['default_value']

            if value:
                if column_raw.get('delimiter'):
                    if column_raw.get('data_type'):
                        if column_raw['data_type'] not in valid_cx_data_types:
                            raise Exception('data_type: ' + column_raw['data_type'] + ' is not valid')
                        value = value.split(column_raw.get('delimiter'))
                        value = [entry.strip() for entry in value]
                        value = data_to_type(value, column_raw['data_type'])
                        if not type_temp.startswith('list'):
                            type_temp = 'list_of_' + type_temp
                    else:
                        if not isinstance(value, str):
                            value = str(value)
                        value = value.split(column_raw.get('delimiter'))
                        value = [entry.strip() for entry in value]
                        type_temp = 'list_of_string'

                    if column_raw.get('value_prefix'):
                        value_list_temp = []
                        for value_item in value:
                            value_temp = column_raw.get('value_prefix') + ":" + str(value_item)
                            value_list_temp.append(value_temp)
                        value = value_list_temp
                else:
                    if column_raw.get('data_type'):
                        if column_raw['data_type'] not in valid_cx_data_types:
                            raise Exception('data_type: ' + column_raw['data_type'] + ' is not valid')

                        value = data_to_type(value, column_raw['data_type'])
                        if value is None:
                            return ''

                    if column_raw.get('value_prefix'):
                        value = column_raw.get('value_prefix') + ":" + str(value)

                if column_raw.get('attribute_name'):
                    nice_cx_builder.add_node_attribute(node_element, column_raw['attribute_name'], value, type=type_temp)
                else:
                    nice_cx_builder.add_node_attribute(node_element, column_raw['column_name'], value, type=type_temp)


def add_edge_attributes(nice_cx_builder, edge_id, load_plan, row):
    if load_plan.get('property_columns'):
        for column_raw_temp in load_plan['property_columns']:
            if isinstance(column_raw_temp, dict):
                column_raw = column_raw_temp
            else:
                if '::' in column_raw_temp:
                    column_split = column_raw_temp.split('::')
                    if len(column_split) > 1:
                        column_raw = {
                            'column_name': column_split[0],
                            'attribute_name': column_split[0],
                            'data_type': column_split[1]
                        }
                    else:
                        column_raw = {
                            'column_name': column_raw_temp,
                            'attribute_name': column_raw_temp
                        }
                else:
                    column_raw = {
                        'column_name': column_raw_temp,
                        'attribute_name': column_raw_temp
                    }

            type_temp = column_raw.get('data_type')

            value = None

            if column_raw.get('column_name'):
                value = row.get(column_raw['column_name'])

            if pd.isnull(value) or value == 'None':
                if column_raw.get('default_value'):
                    value = column_raw['default_value']
                else:
                    continue

            if value is not None:
                dt = None

                if column_raw.get('delimiter'):
                    if column_raw.get('data_type'):
                        dt = str(column_raw.get('data_type'))
                        if dt not in valid_cx_data_types:
                            raise Exception('data_type: ' + dt + ' is not valid')

                        value = value.split(column_raw.get('delimiter'))
                        value = [entry.strip() for entry in value]
                        value = data_to_type(value, dt)
                        if not type_temp.startswith('list'):
                            type_temp = 'list_of_' + type_temp
                    else:
                        value = value.split(column_raw.get('delimiter'))
                        value = [entry.strip() for entry in value]
                        type_temp = 'list_of_string'

                    if column_raw.get('value_prefix'):
                        value_list_temp = []
                        for value_item in value:
                            value_temp = column_raw.get('value_prefix') + ":" + str(value_item)
                            value_list_temp.append(value_temp)
                        value = value_list_temp
                else:
                    if column_raw.get('data_type'):
                        dt = str(column_raw.get('data_type'))
                        if dt not in valid_cx_data_types:
                            raise Exception('data_type: ' + dt + ' is not valid')

                        value = data_to_type(value, dt)

                    if column_raw.get('value_prefix'):
                        value = column_raw.get('value_prefix') + ":" + str(value)

                if value is None:
                    logger.debug('Value is None, skipping edge attribute'
                                 'edge id => ' + str(edge_id) +
                                 ' name => ' + str(column_raw) +
                                 ' values => ' + str(value) +
                                 ' type => ' + str(type_temp))

                    continue

                if column_raw.get('attribute_name'):
                    nice_cx_builder.add_edge_attribute(property_of=edge_id, name=column_raw['attribute_name'],
                                                       values=value, type=type_temp)
                else:
                    nice_cx_builder.add_edge_attribute(property_of=edge_id, name=column_raw['column_name'],
                                                       values=value, type=type_temp)


def data_to_type(data, data_type):
    return_data = None
    try:
        if(type(data) is str):
            data = data.replace('[', '').replace(']','')
            if('list_of' in data_type):
                data = data.split(',')

        if data_type == "boolean":
            if(type(data) is str):
                return_data = data.lower() == 'true'
            else:
                return_data = bool(data)
        elif data_type == "byte":
            return_data = str(data).encode()
        elif data_type == "char":
            return_data = str(data)
        elif data_type == "double":
            return_data = float(data)
        elif data_type == "float":
            return_data = float(data)
        elif data_type == "integer":
            return_data = int(data)
        elif data_type == "long":
            return_data = int(data)
        elif data_type == "short":
            return_data = int(data)
        elif data_type == "string":
            return_data = str(data)
        elif data_type == "list_of_boolean":
            # Assumption: if the first element is a string then so are the rest...
            if(type(data[0]) is str):
                return_data = [s.lower() == 'true' for s in data]
            else:
                return_data = [bool(s) for s in data]
        elif data_type == "list_of_byte":
            return_data = [bytes(s) for s in data]
        elif data_type == "list_of_char":
            return_data = [str(s) for s in data]
        elif data_type == "list_of_double":
            return_data = [float(s) for s in data]
        elif data_type == "list_of_float":
            return_data = [float(s) for s in data]
        elif data_type == "list_of_integer":
            return_data = [int(s) for s in data]
        elif data_type == "list_of_long":
            return_data = [int(s) for s in data]
        elif data_type == "list_of_short":
            return_data = [int(s) for s in data]
        elif data_type == "list_of_string":
            return_data = [str(s) for s in data]
        else:
            return_data = str(data)
    except Exception as err2:
        return_data = None

    return return_data


def find_or_create_node(nice_cx, name, represents, node_lookup):
    if node_lookup.get(represents):
        return nice_cx.nodes.get(node_lookup.get(represents))
    else:
        #new_node_id = nice_cx.create_node(node_name=name, node_represents=represents)
        new_node_id = nice_cx.create_node(node_name=name, node_represents=represents)

        node_lookup[represents] = new_node_id
        return nice_cx.nodes.get(new_node_id)

if __name__ == '__main__':
    print('made it')




"""A setuptools based setup module.
See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
import os
import re


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

with open(os.path.join('ndexutil', '__init__.py')) as ver_file:
    for line in ver_file:
        if line.startswith('__version__'):
            version=re.sub("'", "", line[line.index("'"):])


setup(
    name='ndexutil',
    version=version,
    description='Unsupported NDEx utilities',
    long_description=readme + '\n\n' + history,

    # The project's main homepage.
    url='https://github.com/ndexbio/ndexutils',

    # Author details
    author='The NDEx Project',
    author_email='contact@ndexbio.org',

    # Choose your license
    license='BSD',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        'Natural Language :: English',
        # Indicate who your project is intended for
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Information Analysis',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'Topic :: Scientific/Engineering :: Medical Science Apps.',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: BSD License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3.6'
    ],

    # What does your project relate to?
    keywords='network analysis biology',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=[]),
    data_files=[('schema', ['ndexutil/tsv/loading_plan_schema.json'])],
    install_requires = [
        'ndex2>=3.1.0a1,<=4.0.0',
        'requests',
        'requests_toolbelt',
        'networkx',
        'urllib3>=1.16',
        'pandas',
        'mygene',
        'enum34',
        'pandas',
        'enum; python_version == "2.6" or python_version=="2.7"',
        'jsonschema',
        'biothings_client'
    ],
    test_suite='tests',
    test_requires=[
        'requests-mock',
        'mock'
    ],
    include_package_data=True
)






# -*- coding: utf-8 -*-


class NDExUtilError(Exception):
    """
    Base exception for all errors originating from ndexutil
    """
    pass


class ConfigError(NDExUtilError):
    """
    Raised if there is an error with configuration file
    """
    pass
